@@ -54,7 +54,7 @@ var PolicyState = {
       if (hints.contentType === 6) {
         let origin = hints.requestOrigin;
         if (origin && origin.schemeIs("moz-nullprincipal")) {
-          if (/\n(?:handleCommand@chrome:\/\/[\w/-]+\/urlbarBindings\.xml|.*?@chrome:\/\/noscript\/content\/noscriptBM\.js):\d+\n/
+          if (/\n(?:handleCommand@chrome:\/\/[\w/-]+\/urlbarBindings\.xml|.*?@chrome:\/\/noscript\/content\/noscriptBM\.js)(?::\d+)+\n/
             .test(new Error().stack)) {
             hints.requestOrigin = ABE.BROWSER_URI;
           } else  if (hints.context.docShell) {
@@ -150,7 +150,7 @@ PolicyHints.prototype = (function() {
             v = null;
           }
           if (v) {
-            this.__defineGetter__(i, function() v.get());
+            this.__defineGetter__(i, function() { try { return v.get() } catch (e) {} return null } );
           } else {
             this.args[i] = v;
           }
@@ -298,7 +298,13 @@ const MainContentPolicy = {
       unwrappedLocation = IOUtil.unwrapURL(aContentLocation);
       scheme = unwrappedLocation.scheme;
       
-    
+      if (scheme === "file") {
+        let principal =  aContext && (aContext.nodePrincipal ||
+                                      aContext.document && aContext.document.nodePrincipal);
+        if (!ns.checkLocalLink(unwrappedLocation, principal, true)) {
+          return this.reject("Local File Link", arguments);
+        }
+      }
       
       var isHTTP = scheme === "http" || scheme === "https";
       
@@ -314,7 +320,7 @@ const MainContentPolicy = {
         }
         
         if (aRequestOrigin &&
-            !(aContentType === 4 && this._bug677643)
+            !(aContentType === 4 && Bug.$677643)
             ) {
           
           HTTPS.forceURI(unwrappedLocation, null, aContext);
@@ -335,9 +341,10 @@ const MainContentPolicy = {
                   this.isCachedObjectMime(aMimeTypeGuess) &&
                   !(aContext.offsetWidth && aContext.offsetHeight) &&
                   this.getPref("allowCachingObjects") &&
-                  aRequestOrigin && this.isJSEnabled(this.getSite(aRequestOrigin.spec)) &&
+                  let (win = aContext.ownerDocument.defaultView)
+                  aRequestOrigin && this.isJSEnabled(this.getSite(aRequestOrigin.spec), win) &&
                   !this.pluginForMime(aMimeTypeGuess) &&
-                  (aMimeTypeGuess.indexOf("css") > 0 || this.isJSEnabled(this.getSite(aContentLocation.spec)))
+                  (aMimeTypeGuess.indexOf("css") > 0 || this.isJSEnabled(this.getSite(aContentLocation.spec), win))
                  ) {
                 return CP_OK;
               }
@@ -380,7 +387,16 @@ const MainContentPolicy = {
             return CP_OK;
             
           return this.reject("Ping", arguments);
+        
+         case 17: // CSP report, avoid exfiltration on untrusted resources
+          if (this.jsEnabled ||
+              this.isJSEnabled(this.getSite(aContentLocation.spec)) &&
+              aRequestOrigin && this.isJSEnabled(this.getSite(aRequestOrigin.spec))
+            )
+            return CP_OK;
             
+          return this.reject("CSP report", arguments);
+        
         case 2:
           forbid = isScript = true;
           break;
@@ -431,7 +447,7 @@ const MainContentPolicy = {
           
           case 15: // media
           if (aContentType === 15) {
-              if (aRequestOrigin && !this.isJSEnabled(this.getSite(aRequestOrigin.spec))) {
+              if (aRequestOrigin && !this.isJSEnabled(this.getSite(aRequestOrigin.spec), aContext.ownerDocument.defaultView)) {
               // let's wire poor man's video/audio toggles if JS is disabled and therefore controls are not available
               this.delayExec(function() {
                 aContext.addEventListener("click", function(ev) {
@@ -628,7 +644,8 @@ const MainContentPolicy = {
        
         let scriptElement;
         if (aContentType === 2) { // "real" JavaScript include
-          if (originSite && !this.isJSEnabled(originSite) &&
+          if (!(this.cascadePermissions || this.globalHttpsWhitelist) &&
+              originSite && !this.isJSEnabled(originSite, contentDocument.window) &&
               isHTTP && httpOrigin) {
             // JavaScript-disabled page with script inclusion
             this.syncUI(contentDocument);
@@ -662,16 +679,17 @@ const MainContentPolicy = {
           }
         } else isScript = scriptElement = false;
 
-        if (forbid) forbid = !this.isJSEnabled(locationSite);
-        if (forbid && this.ignorePorts && /:\d+$/.test(locationSite))
-          forbid = !(this.isJSEnabled(locationSite.replace(/:\d+$/, '')) && this.autoTemp(locationSite));
+        if (forbid) {
+          let doc = aContext.ownerDocument || aContext;
+          let win = doc && doc.defaultView;
+          forbid = !this.isJSEnabled(locationSite, win);
+          if (forbid && this.ignorePorts && /:\d+$/.test(locationSite))
+            forbid = !(this.isJSEnabled(locationSite.replace(/:\d+$/, '')) && this.autoTemp(locationSite));
+        }
 
         if ((untrusted || forbid) && scheme !== "data") {
           if (scriptElement) {
-            if (ScriptSurrogate.apply(contentDocument, locationURL)) {
-              let surrogates = this.getExpando(contentDocument, "surrogates", {});
-              surrogates[locationURL] = true;
-            }
+            ScriptSurrogate.replaceScript(aContext);
           }
           
           this.syncUI(contentDocument);
@@ -681,8 +699,6 @@ const MainContentPolicy = {
           
           if (scriptElement) {
             
-            ScriptSurrogate.apply(contentDocument, locationURL, "<");
-            
             if (this.executingJSURL(contentDocument.defaultView.top.document) &&
                 !this.jsPolicySites.matches(this.getSite(contentDocument.defaultView.location.href))) {
               this.bookmarkletImport(aContext, locationURL);
@@ -760,14 +776,20 @@ const MainContentPolicy = {
         originURL = originURL || (aRequestOrigin && aRequestOrigin.spec);
         originSite = originSite || this.getSite(originURL);
         
+        let win = aContext && (
+          aContext.ownerDocument
+          ? aContext.ownerDocument.defaultView
+          : aContext.document ? aContext : aContext.defaultView
+        );
+        
         let jsRx = /^(?:javascript|data):/;
         
         let originOK = originSite 
-          ? this.isJSEnabled(originSite) 
+          ? this.isJSEnabled(originSite, win) 
           : jsRx.test(originURL); // if we've got such an origin, parent should be trusted
         
         let locationOK = locationSite 
-              ? this.isJSEnabled(locationSite) 
+              ? this.isJSEnabled(locationSite, win)
               : jsRx.test(locationURL) && originOK // use origin for javascript: or data:
         ;
         
@@ -788,6 +810,11 @@ const MainContentPolicy = {
       
       if (forbid) forbid = !(aContentLocation.schemeIs("file") && aRequestOrigin && aRequestOrigin.schemeIs("resource")); // fire.fm work around
       
+      if (forbid && this.cascadePermissions && !this.contentBlocker) {
+        let principal = aContext.ownerDocument && aContext.ownerDocument.defaultView.top.document.nodePrincipal;
+        forbid = untrusted || !this.isJSEnabled(this.getSite(principal.origin), aContext.ownerDocument.defaultView); 
+      }
+      
       if (forbid) {
         
         if (!originSite) originSite = this.getSite(originURL || (originURL = aRequestOrigin && aRequestOrigin.spec || ""));
@@ -854,7 +881,6 @@ const MainContentPolicy = {
       
       if (isHTTP) PolicyState.save(unwrappedLocation, arguments);
       
-      
     }
     return CP_OK;
   },
@@ -868,8 +894,9 @@ const MainContentPolicy = {
 }
 
 var PolicyUtil = {
-  unsupportedXSL: !("TYPE_XSLT" in Ci.nsIContentPolicy),
+  supportsXSL: ("TYPE_XSLT" in Ci.nsIContentPolicy),
   isXSL: function(ctx) {
     return ctx && !(ctx instanceof Ci.nsIDOMHTMLLinkElement || ctx instanceof Ci.nsIDOMHTMLStyleElement || ctx instanceof Ci.nsIDOMHTMLDocument);
-  }
+  },
+  
 };
