@@ -1,5 +1,4 @@
-const Cc = Components.classes;
-const Ci = Components.interfaces;
+// Cc, Ci, Cu should be already defined by tree-copy.js
 
 var ns = noscriptUtil.service;
 
@@ -8,6 +7,7 @@ var nsopt = {
 
   dom2: /^(?:http[s]?|file):\/\/([^\.\?\/#,;:\\\@]+(:?\.[^\.\?\/#,;:\\\@]+$|$))/,
   utils: null,
+  whitelistURLs: [],
   init: function() {
 
     if(ns.uninstalling) { // this should never happen! 
@@ -23,7 +23,7 @@ var nsopt = {
     abeOpts.init();
 
     var locked = ns.locked;
-    for each (var widget in ["urlText","urlList", "jsglobal", "addButton", "removeButton", "importButton", "exportButton"]) {
+    for each (var widget in ["urlText","urlListDisplay", "jsglobal", "addButton", "removeButton", "importButton", "exportButton"]) {
       this[widget] = $(widget);
       if(locked) this[widget].disabled = true;
     }
@@ -39,6 +39,8 @@ var nsopt = {
     this.untrustedSites = ns.untrustedSites.clone();
     this.tempSites = ns.tempSites.clone();
     this.gTempSites = ns.gTempSites.clone();
+
+    this.urlListDisplay.boxObject.QueryInterface(Ci.nsITreeBoxObject);
     this.populateUrlList();
     
     this.jsglobal.checked = ns.jsEnabled;
@@ -113,6 +115,10 @@ var nsopt = {
     if (!ns.placesSupported)
       $("opt-placesPrefs").setAttribute("hidden", "true");
     
+    if (!ns.supportsCAPS) {
+      $("opt-allowClipboard").setAttribute("collapsed", "true");
+    }
+    
     this.initExtraButtons();
     
     this.addButton.setAttribute("enabled", "false");
@@ -273,11 +279,11 @@ var nsopt = {
     return true;
   },
   
-  urlListChanged: function() {
-    const selectedItems = this.urlList.selectedItems;
+  urlListChanged: function(dontUpdate) {
+    const selectedItems = noscriptTreeCc.getSelectedItems(this.urlListDisplay, this.whitelistURLs);
     var removeDisabled = true;
     for(var j = selectedItems.length; j-- > 0;) {
-      if(selectedItems[j].getAttribute("disabled") != "true") {
+      if(!selectedItems[j].mandatory) {
         removeDisabled = false;
         break;
       }
@@ -286,13 +292,14 @@ var nsopt = {
     $("revokeButton")
       .setAttribute("disabled", this.tempRevoked || 
           !(this.tempSites.sitesString || this.gTempSites.sitesString || ns.objectWhitelistLen));
+    if (!dontUpdate) nsWhitelistTreeView.updateTree();
     this.urlChanged();
   },
   
   openInfo: function(ev) {
     if (ev.button === 1) {
       setTimeout(function() {
-        const selectedItems = nsopt.urlList.selectedItems;
+        const selectedItems = noscriptTreeCc.getSelectedItems(nsopt.urlListDisplay, nsopt.whitelistURLs);
         const domains = [];
         for (let j = selectedItems.length; j-- > 0;) {
           let site = selectedItems[j].value;
@@ -303,6 +310,10 @@ var nsopt = {
       }, 0); // delayed to let middle-click autoselect the underlying item
     }
   },
+
+  copyUrlListSel: function() {
+    noscriptTreeCc.doCopy(this.urlListDisplay, "", " ");
+  },
   
   urlChanged: function() {
     var url = this.urlText.value;
@@ -329,20 +340,17 @@ var nsopt = {
   },
   
   ensureVisible: function(site) {
-    var item;
-    const ul = this.urlList;
-    for(var j = ul.getRowCount(); j-- > 0;) {
-      if((item = ul.getItemAtIndex(j)).getAttribute("value") == site) {
-        ul.ensureElementIsVisible(item);
+    for(var j = nsWhitelistTreeView.rowCount; j-- > 0;) {
+      if(nsWhitelistTreeView.getCellText(j) == site) {
+        this.urlListDisplay.boxObject.ensureRowIsVisible(j);
       }
     }
   },
-  
+
   populateUrlList: function() {
     const policy = this.trustedSites;
     const sites = this.trustedSites.sitesList;
-    const ul = this.urlList;
-    for(var j = ul.getRowCount(); j-- > 0; ul.removeItemAt(j));
+    this.whitelistURLs = [];
     const dom2 = this.dom2;
     var site, item;
     var match, k, len;
@@ -355,15 +363,17 @@ var nsopt = {
       if((match = site.match(dom2)) && policy.matches(item = match[1])) 
         continue;
       
-      item = ul.appendItem(site, site);
+      item = {value:site};
       if(ns.isMandatory(site)) { 
-        item.setAttribute("disabled", "true");
+        item.mandatory = true;
       }
-      item.style.fontStyle = (site in tempMap) ? "italic" : "normal";
+      item.temp = site in tempMap;
+      this.whitelistURLs.push(item);
     }
+    this.urlListDeselectAll();
     this.urlListChanged();
   },
-  
+
   allow: function() {
     const site = ns.getSite(this.urlText.value);
     this.trustedSites.add(site);
@@ -377,11 +387,10 @@ var nsopt = {
   },
   
   remove: function() {
-    const ul = this.urlList;
-    const selectedItems = ul.selectedItems;
-    var visIdx = ul.getIndexOfFirstVisibleRow();
-    var lastIdx = visIdx + ul.getNumberOfVisibleRows();
-   
+    const ul = this.urlListDisplay;
+    const selectedItems = noscriptTreeCc.getSelectedItems(ul, this.whitelistURLs);
+    var visIdx = ul.boxObject.getFirstVisibleRow();
+    var lastIdx = visIdx + ul.boxObject.getPageLength();
     
     
     
@@ -398,25 +407,14 @@ var nsopt = {
     this.tempSites.remove(removed, true, true); // see noscriptService#eraseTemp()
     this.gTempSites.remove(removed, true, true);
       
-      
-    if(selectedItems.length == 1) {
-      if(removed.length == 1) {
-        ul.removeItemAt(ul.getIndexOfItem(selectedItems[0]));  
-      }
-      return;
-    }
-
     this.populateUrlList();
-    try {
-      var rowCount = ul.getRowCount();
-      if(rowCount > lastIdx) {
-        ul.scrollToIndex(visIdx);
-      } else {
-        ul.ensureIndexIsVisible(rowCount - 1);
-      } 
-    } catch(e) {}
+
   },
-  
+
+  urlListDeselectAll: function() {
+    this.urlListDisplay.view.selection.clearSelection();
+  },
+
   tempRevoked: false,
   revokeTemp: function() {
     this.trustedSites.remove(this.tempSites.sitesList, true, true);
@@ -570,7 +568,7 @@ var abeOpts = {
     if (current && this.dirty) {
       
       let name = current.value;
-      let source = $("abeRuleset-text").value;
+      let source = $("abeRuleset-text-container").selectedPanel.value;
       let ruleset = ABE.createRuleset(name, source);
       if (ruleset.errors && this.dirty) {
         this.dirty = null;
@@ -601,7 +599,13 @@ var abeOpts = {
     try {
       const map = {__proto__: null};
       var l = this.list;
-      for(var j = l.getRowCount(); j-- > 0; l.removeItemAt(j));
+      for(var j = l.getRowCount(); j-- > 0; l.removeItemAt(j)) {
+        try {
+          let rc = $("abeRuleset-text-container");
+          rc.removeChild(rc.lastChild);
+        }
+        catch (e) { /* no textboxes present to remove, ignore */ }
+      }
       var rulesets = ABE.rulesets;
       var selItem = null;
       if (rulesets) {
@@ -618,6 +622,17 @@ var abeOpts = {
             i.className = "noscript-error";
             this.errors = rs.errors;
           }
+          let textbox = document.createElement("textbox");
+          let textboxAttributes = {
+            "flex":"5",
+            "multiline":"true",
+            "wrap":"off",
+            "onchange":"abeOpts.changed()",
+            "oninput":"abeOpts.input(this)",
+            "value":rs.source
+          };
+          for (let a in textboxAttributes) { textbox.setAttribute(a, textboxAttributes[a]); }
+          $("abeRuleset-text-container").appendChild(textbox);
         }
       }
       this._map = map; 
@@ -665,19 +680,20 @@ var abeOpts = {
     
     $("abeEnable-button").disabled = ! ($("abeDisable-button").disabled = !rs || rs.disabled);
     $("abeRefresh-button").disabled = this.list.getRowCount() == 0;
+    $("abeRuleset-text-container").setAttribute("selectedIndex", this.list.selectedIndex);
     
-    var text = $("abeRuleset-text");
+    var text = $("abeRuleset-text-container").selectedPanel;
     text.className = selItem && selItem.className || '';
     text.disabled = !selItem || selItem.disabled;
     text.value = rs && rs.source;
     
     text = $("abeRuleset-errors");
     if (rs && rs.errors) {
-      text.hidden = false;
+      this.ShowHideABEError(false);
       text.value = rs.errors.join("\n");
     }
     else {
-      text.hidden = true;
+      this.ShowHideABEError(true);
       text.value = "";
     }
   },
@@ -685,6 +701,12 @@ var abeOpts = {
   refresh: function() {
     ABE.refresh();
   },
+ 
+  ShowHideABEError: function(hidden) {
+    for each (let n in document.getElementsByClassName("abe-error-element")) {
+      n.hidden = hidden;
+    }
+  },
   
   toggle: function(enabled) {
     var selItem = this.list.selectedItem;
@@ -699,4 +721,44 @@ var abeOpts = {
     this.sync();
   }
   
-}
\ No newline at end of file
+}
+
+var nsWhitelistTreeView = {
+  rowCount: nsopt.whitelistURLs.length,
+  getCellText: function(r, c) { return nsopt.whitelistURLs[r].value; },
+  setTree: function(treebox){ this.treebox = treebox; },
+  isContainer: function(row){ return false; },
+  isSeparator: function(row){ return false; },
+  isSorted: function(){ return true; },
+  getLevel: function(row){ return 0; },
+  getImageSrc: function(row,col){ return null; },
+  getRowProperties: function(row,props) {
+    // ??? need to set multicol manually for some reason
+    if (props) {
+      let aserv=Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
+      props.AppendElement(aserv.getAtom("multicol"));
+    }
+    else { return "multicol" }
+  },
+  getCellProperties: function(row,col,props) {
+    var psl = [];
+    if (nsopt.whitelistURLs[row].temp) psl.push("temp");
+    if (nsopt.whitelistURLs[row].mandatory) psl.push("mandatory");
+    if (psl.length == 0) return;
+    if (props) {
+      let aserv=Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
+      for each (let s in psl) props.AppendElement(aserv.getAtom(s));
+    }
+    else { return psl.join(" ") }
+  },
+  getColumnProperties: function(colid,col,props){},
+  cycleHeader: function(col){},
+
+  // Custom properties and methods
+  updateTree: function() {
+    var r = nsopt.urlListDisplay.boxObject.getFirstVisibleRow();
+    nsWhitelistTreeView.rowCount = nsopt.whitelistURLs.length;
+    $("urlListDisplay").view = nsWhitelistTreeView;
+    nsopt.urlListDisplay.boxObject.scrollToRow(r);
+  },
+};
