@@ -7,6 +7,7 @@ const Ci = Components.interfaces;
 return noscriptUtil.service ? {
 
   ns: noscriptUtil.service,
+  recentlyBlocked: [],
   
   getString: function(key, parms) {
     return noscriptUtil.getString(key, parms);
@@ -141,6 +142,9 @@ return noscriptUtil.service ? {
             case "open":
             case "showing":
               popup.hidePopup();
+              if (popup.state !== "closed") try {
+                popup. boxObject.QueryInterface(Components.interfaces.nsIPopupBoxObject).hidePopup()
+              } catch(e) {}
               break;
           }
         }
@@ -351,57 +355,88 @@ return noscriptUtil.service ? {
     return tip;
   },
   
-  _popupsInitialized: false,
-  initPopups: function() {
-    
-    const sticky = this.stickyUI; // early init
-
-    const popup = $("noscript-status-popup");
-    if (!popup) return; // Fennec?
-    
-    const install = !this._popupsInitialized;
-    this._popupsInitialized = true;
-    
-    const tbb = $("noscript-tbb");
-    if (tbb) {
-      tbb.setAttribute("type", this.hoverUI ? "button" : this.ns.getPref("toolbarToggle") ? "menu-button" : "menu");
-    }
-
-    const buttons = [tbb, $("noscript-statusLabel")];
-    let statusIcon = $("noscript-statusIcon");
-    if ($("addon-bar")) {
-      // Fx 4 or above
-      if (install) {  
-        window.addEventListener("aftercustomization", function(ev) {
-          noscriptOverlay.initPopups();
-        }, false);
-      }
-      if (statusIcon) statusIcon.parentNode.removeChild(statusIcon);
-    } else {
-      // Fx 3.6.x or below
-      if (install) {  
-        let btcd = window.BrowserToolboxCustomizeDone;
-        if (btcd) window.BrowserToolboxCustomizeDone = function(done) {
-          btcd(done);
-          if (done) noscriptOverlay.initPopups();
+  _customizableUIListener: {
+    onWidgetAfterDOMChange: function(aWidget) {
+      for each(let b in ['noscript-tbb', 'noscript-statusLabel']) {
+        if(b == aWidget.id) {
+          window.setTimeout(function() { noscriptOverlay.initPopups(); }, 0);
+          return;
         }
       }
-      
-      buttons.push(statusIcon);
     }
-    // copy status bar menus
-    for each(let button in buttons) {
-      if (!button) continue;
-      let localPopup = button.firstChild;
-      if (!(localPopup && /popup/.test(localPopup.tagName))) {
-        localPopup = popup.cloneNode(true);
-        localPopup.id  = button.id + "-popup";
-        button.insertBefore(localPopup, button.firstChild);
-        if (!sticky) localPopup._context = true;
+  },
+        
+  _popupsInitialized: false,
+  _initPopupsRecursion: false,
+  
+  initPopups: function() {
+    noscriptOverlay.ns.log("InitPopup: " + (this._initPopupsRecursion + ", " + this._popupsInitialized) + " - " + new Error().stack);
+    if (this._initPopupsRecursion) return;
+    
+    this._initPopupsRecursion = true;
+    
+    try {
+      
+    
+      const sticky = this.stickyUI; // early init
+  
+      const popup = $("noscript-status-popup");
+      if (!popup) return; // Fennec?
+      
+      const install = !this._popupsInitialized;
+      this._popupsInitialized = true;
+      
+      const tbb = $("noscript-tbb");
+      if (tbb) {
+        tbb.setAttribute("type", this.hoverUI ? "button" : this.ns.getPref("toolbarToggle") ? "menu-button" : "menu");
       }
-      localPopup.position = /^(?:addon|status)-bar$/.test(button.parentNode.id)
-        ? "before_start"
-        : "after_start";
+  
+      const buttons = [tbb, $("noscript-statusLabel")];
+      
+      if (install && "CustomizableUI" in window) {
+        CustomizableUI.addListener(this._customizableUIListener);
+      }
+      
+      let statusIcon = $("noscript-statusIcon");
+      
+      if ($("addon-bar")) {
+        // Fx 4  till Austrails
+        if (install &&  !("CustomizableUI" in window)) {  
+          window.addEventListener("aftercustomization", function(ev) {
+            noscriptOverlay.initPopups();
+          }, false);
+        }
+        if (statusIcon) statusIcon.parentNode.removeChild(statusIcon);
+      } else {
+        // Fx 3.6.x or below
+        if (install) {  
+          let btcd = window.BrowserToolboxCustomizeDone;
+          if (btcd) window.BrowserToolboxCustomizeDone = function(done) {
+            btcd(done);
+            if (done) noscriptOverlay.initPopups();
+          }
+        }
+        
+        buttons.push(statusIcon);
+      }
+      // copy status bar menus
+      for each(let button in buttons) {
+        if (!button) continue;
+        let localPopup = button.firstChild;
+        if (!(localPopup && /popup/.test(localPopup.tagName))) {
+          localPopup = popup.cloneNode(true);
+          localPopup.id  = button.id + "-popup";
+          button.insertBefore(localPopup, button.firstChild);
+          if (!sticky) localPopup._context = true;
+        }
+        if (!this._mustReverse(localPopup)) {
+          localPopup.position = /(?:addon|status)/.test(button.parentNode.id)
+          ? "before_start"
+          : "after_start";
+        }
+      }
+    } finally {
+      this._initPopupsRecursion = false;
     }
   },
   
@@ -438,17 +473,23 @@ return noscriptUtil.service ? {
   },
  
   _currentPopup: null,
-  
-  prepareMenu: function(popup, sites) {
-    let mustReverse;
-    
-    if (popup.id === "noscript-tbb-popup") {
-      mustReverse = popup.position === "after_start";
-      if (/\bnoscript-(?:about|options)\b/.test(popup.lastChild.className)) {
+  _mustReverse: function(popup) {
+    if (popup.id !== "noscript-tbb-popup") return false;
+    if (/\bnoscript-(?:about|options)\b/.test(popup.lastChild.className))  {
       // already reversed: we need it straight to populate
-        this.reverse(popup);
-      }
-    } else mustReverse = false;
+      this.reverse(popup);
+    }
+    let upper;
+    try {
+      upper = popup.parentNode.boxObject.screenY < screen.availHeight / 2;
+    } catch(e) {
+      upper = false;
+    }
+    popup.position = upper ? "after_start" : "before_start";
+    return upper;
+  },
+  prepareMenu: function(popup, sites) {
+    let mustReverse = this._mustReverse(popup);
 
     const ns = this.ns;
     const sticky = popup.getAttribute("sticky") == "true";
@@ -470,7 +511,9 @@ return noscriptUtil.service ? {
     
     const global = ns.jsEnabled;
     const blockUntrusted = global && ns.alwaysBlockUntrustedContent;
-
+    const cascadePermissions = ns.cascadePermissions;
+    const globalHttps = ns.globalHttpsWhitelist;
+    
     var seps = { insert: null, stop: null, global: null, untrusted: null };
     {
       let allSeps = popup.getElementsByTagName("menuseparator");
@@ -668,7 +711,9 @@ return noscriptUtil.service ? {
       }
       
 
-      enabled = !!matchingSite;
+      enabled = !!(matchingSite ||
+                   globalHttps && ns.isGlobalHttps(content, site) && (matchingSite = site)
+                );
       
       let showInMain = embedOnlySites
         ? embedOnlySites.indexOf(site) === -1 || hideUntrustedPlaceholder && enabled : true;
@@ -756,10 +801,10 @@ return noscriptUtil.service ? {
       let max = ns.getPref("recentlyBlockedCount");
       let dejaVu = [],
           count = 0,
-          recent = ns.recentlyBlocked,
+          recent = this.recentlyBlocked,
           current = false;
       
-      let tooltip = noscriptOverlay.getSiteTooltip(false, !!ns.getPref("siteInfoProvider"));
+      let tooltip = this.getSiteTooltip(false, !!ns.getPref("siteInfoProvider"));
       
       for (let j = recent.length; j-- > 0;) {
         
@@ -768,7 +813,8 @@ return noscriptUtil.service ? {
         
         if (!s || sites.indexOf(s) > -1) continue;
         
-        let jsEnabled = ns.isJSEnabled(s);
+        let ghEnabled = globalHttps && ns.isGlobalHttps(content, s);
+        let jsEnabled = ghEnabled || ns.isJSEnabled(s);
         
         if (jsEnabled && (!ns.contentBlocker || ns.isAllowedObject("!", "*", s)))
           continue;
@@ -825,6 +871,7 @@ return noscriptUtil.service ? {
       menuSites = menuGroups[mgCount];
       isTop = menuSites.isTop;
       enabled = menuSites.enabled;
+     
       let showInMain = menuSites.showInMain;
       
       if (untrustedFrag && untrustedFrag.firstChild) {
@@ -837,8 +884,12 @@ return noscriptUtil.service ? {
         
       while (scount-- > 0) {
         menuSite = menuSites[scount];
+        let ghEnabled = globalHttps && ns.isGlobalHttps(content, menuSite);
         
         untrusted = !enabled && (blockUntrusted || ns.isUntrusted(menuSite));
+        
+        let cascaded = cascadePermissions && !isTop;
+        
         if (untrusted) {
           untrustedCount++;
           showInMain = true;
@@ -846,7 +897,7 @@ return noscriptUtil.service ? {
         else if (!enabled)
           unknownCount++;
         
-        parent = (untrusted && showUntrusted) ? untrustedFrag : mainFrag;
+        parent = showUntrusted && untrusted ? untrustedFrag : mainFrag;
         if (!parent) continue;
                 
         domain = isTop && docJSBlocked ? "[ " + menuSite + " ]" : menuSite;
@@ -878,17 +929,19 @@ return noscriptUtil.service ? {
     
         node.setAttribute("class", cssClass + (enabled ? " noscript-forbid" : " noscript-allow"));
         
-        if ((showPermanent || enabled) && !(global && enabled) && showInMain) 
+        if ((showPermanent || enabled) && !((global || cascaded || ghEnabled) && enabled) &&
+            showInMain && !(cascaded && parent !== untrustedFrag)) 
           parent.appendChild(node);
       
         if (!disabled) {
-          if (showTemp && !(enabled || blurred) && showInMain) {
+          if (showTemp && !(enabled || blurred || cascaded) && showInMain) {
             extraNode = node.cloneNode(false);
             extraNode.setAttribute("label", this.getString("allowTemp", [domain]));
             extraNode.setAttribute("class", cssClass + " noscript-temp noscript-allow");
             parent.appendChild(extraNode);
           }
-          if (((showUntrusted && untrustedMenu || showDistrust) && !(domain in jsPSs.sitesMap) ||
+          if (((showUntrusted && untrustedMenu || showDistrust) && 
+                (cascaded || ghEnabled || !(domain in jsPSs.sitesMap)) ||
                 blockUntrusted && (showUntrusted || showDistrust)
                 ) && !untrusted) {
             parent = (showUntrusted && !blockUntrusted ? untrustedFrag : mainFrag);
@@ -916,7 +969,9 @@ return noscriptUtil.service ? {
     
     // temp allow all this page
     if (!(tempMenuItem.hidden = !(unknownCount && ns.getPref("showTempAllowPage", true)))) {
-      tempMenuItem.setAttribute("tooltiptext", this.allowPage(false, true, sites).join(", "));
+      let allowable = this.allowPage(false, true, sites);
+      if (allowable.length) tempMenuItem.setAttribute("tooltiptext", allowable.join(", "));
+      else tempMenuItem.hidden = true;
     }
 
     
@@ -926,7 +981,9 @@ return noscriptUtil.service ? {
       tempMenuItem.parentNode.insertBefore(node, tempMenuItem);
     }
     if (!(node.hidden = unknownCount == 0 || !ns.getPref("showAllowPage", true))) {
-      node.setAttribute("tooltiptext", this.allowPage(true, true, sites).join(", "));
+       let allowable = this.allowPage(true, true, sites);
+      if (allowable.length) node.setAttribute("tooltiptext", allowable.join(", "));
+      else node.hidden = true;
     }
     
     // "allow page" accelerators
@@ -1157,8 +1214,13 @@ return noscriptUtil.service ? {
     const level = ns.getPref("allowPageLevel", 0) || ns.preferredSiteLevel;
     const trusted = ns.jsPolicySites;
     const tempToPerm = permanent === -1;
-    var site;
+    
+    const topSite = sites.topSite;
+    const cascade = topSite && ns.cascadePermissions;
+    
     for (let j = sites.length; j-- > 0;) {
+      let site = sites[j];
+      if (cascade && topSite !== site)  continue;
       if (tempToPerm) {
         site = trusted.matches(sites[j]);
         if (!(site && ns.isTemp(site)) || ns.isUntrusted(site)) continue;
@@ -1882,6 +1944,8 @@ return noscriptUtil.service ? {
     
     const ns = this.ns;
     const global = ns.jsEnabled;
+    const cascadePermissions = ns.cascadePermissions;
+    const globalHttps = ns.globalHttpsWhitelist;
     const jsPSs = ns.jsPolicySites;
     const untrustedSites = ns.untrustedSites;
 
@@ -1913,6 +1977,7 @@ return noscriptUtil.service ? {
     var topTrusted = false;
     var topUntrusted = false;
 
+    let win = content;
     
     if (global && !ns.alwaysBlockUntrustedContent) {
       lev = "glb";
@@ -1936,10 +2001,10 @@ return noscriptUtil.service ? {
       while (s-- > 0) {
         let url = sites[s];
         let isUntrusted = untrustedSites.matches(url);
-        let site = !isUntrusted && (global ? url : jsPSs.matches(url));
+        let site = !isUntrusted && (global || globalHttps && ns.isGlobalHttps(win, url) ? url : jsPSs.matches(url));
         
         if (url == sites.topSite) {
-          if (site && (!ns.httpStarted || ns.dom.getDocShellForWindow(content).allowJavascript)) topTrusted = true;
+          if (site && (!ns.httpStarted || ns.dom.getDocShellForWindow(win).allowJavascript)) topTrusted = true;
           else {
             site = null;
             if (isUntrusted) topUntrusted = true;
@@ -1966,8 +2031,8 @@ return noscriptUtil.service ? {
       }
       
       allowed = allowedSites.length;
-      lev = (allowed == total && sites.length > 0 && !untrusted) ? (global ? "glb" : "yes")
-            : (allowed == 0 || active == 0) ? (global
+      lev = (allowed === total && sites.length > 0 && !untrusted) ? (global ? "glb" : "yes")
+            : (allowed === 0 || active === 0) ? (global
                                               ? "untrusted-glb" :
                                                 topUntrusted
                                                   ? "untrusted" :
@@ -1975,14 +2040,17 @@ return noscriptUtil.service ? {
             : (untrusted > 0 && !notificationNeeded
                 ? (blockedObjects ? (global ? "glb-emb" : "yu-emb") : global ? "yu-glb" : "yu") 
                 : topTrusted
-                  ? allowed == total - blockedObjects
+                  ? allowed === total - blockedObjects
                       ? (global ? "glb-emb" : "emb")
-                      : "prt"
-                  : ns.docShellJSBlocking == 2
+                      : (cascadePermissions ? "yes" : "prt")
+                  : ns.docShellJSBlocking === 2 || cascadePermissions || ns.restrictSubdocScripting
                       ? "no"
                       : "subprt"
               );
-      notificationNeeded = notificationNeeded && totalAnnoyances > 0;
+      notificationNeeded = notificationNeeded &&
+        (totalAnnoyances > 0 &&
+          (!cascadePermissions || !topTrusted || blockedObjects)
+        );
     }
     
     let message = this.getString(
@@ -2015,7 +2083,7 @@ return noscriptUtil.service ? {
     }
     
     if (notificationNeeded) { // notifications
-      let win = content;
+      
       if (this.notify) {
         this.notificationShow(message,
           this.getIcon(widget), 
@@ -2080,24 +2148,29 @@ return noscriptUtil.service ? {
     this.ns.applyPreset(menulist.selectedItem.value);
   },
   
-  prefsObserver: {
+  observer: {
     ns: noscriptUtil.service,
     QueryInterface: noscriptUtil.service.generateQI([
         Ci.nsIObserver, 
         Ci.nsISupportsWeakReference])
   ,
     observe: function(subject, topic, data) {
-      
-      if (topic == "noscript:sync-ui") {
-        noscriptOverlay.syncUI(subject);
-        return;
-      }
-      
       if (subject == this.ns.caps) {
          noscriptOverlay.syncUI();
          return;
       }
       
+      switch(topic) {
+        case "noscript:sync-ui":
+          noscriptOverlay.syncUI(subject);
+          return;
+        case "browser:purge-session-history":
+          noscriptOverlay.recentlyBlocked = [];
+          return;
+      }
+      
+      // prefs
+      
       switch (data) {
         case "preset":
           if (data == "off") noscriptOverlay.statusIcon.setAttribute("hidden", "true");
@@ -2142,12 +2215,17 @@ return noscriptUtil.service ? {
           noscriptOverlay.initPopups();
           
         break;
+        
       }
     },
     _registered: false,
+    _topics: ["noscript:sync-ui", "browser:purge-session-history"],
     register: function() {
       const ns = this.ns;
-      ns.os.addObserver(this, "noscript:sync-ui", true);
+      const os = ns.os;
+      for each (let t in this._topics){
+        os.addObserver(this, t, true);
+      }
       ns.prefs.addObserver("", this, true);
       ns.caps.addObserver("", this, true);
       const initPrefs = [
@@ -2158,14 +2236,17 @@ return noscriptUtil.service ? {
         "stickyUI.liveReload",
         "hoverUI"
         ];
-      for (var j = 0; j < initPrefs.length; j++) {
-        this.observe(null, null, initPrefs[j]);
+      for each (let p in initPrefs) {
+        this.observe(null, null, p);
       }
       this._registered = true;
     },
     remove: function() {
       const ns = this.ns;
-      ns.os.removeObserver(this, "noscript:sync-ui");
+      const os = ns.os;
+      for each (let t in this._topics){
+        os.removeObserver(this, t);
+      }
       ns.prefs.removeObserver("", this);
       ns.caps.removeObserver("", this);
     }
@@ -2435,7 +2516,7 @@ return noscriptUtil.service ? {
         noscriptOverlay.wrapBrowserAccess();
         let hacks = noscriptOverlay.Hacks;
         hacks.torButton();
-
+        hacks.allowLocalLinks();
         window.setTimeout(function() {
           hacks.pdfDownload();
           Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader)
@@ -2515,7 +2596,7 @@ return noscriptUtil.service ? {
 
      
       noscriptOverlay.shortcutKeys.register();
-      noscriptOverlay.prefsObserver.register();
+      noscriptOverlay.observer.register();
       
       let self = this;
       
@@ -2583,7 +2664,7 @@ return noscriptUtil.service ? {
       window.removeEventListener("pageshow", this.onPageShow, true);
       window.removeEventListener("DOMContentLoaded", this.onContentLoad, false);
 
-      noscriptOverlay.prefsObserver.remove();
+      noscriptOverlay.observer.remove();
       noscriptOverlay.shortcutKeys.remove();
       
      $("contentAreaContextMenu").removeEventListener("popupshowing", this.onMainContextMenu, false);
@@ -2699,14 +2780,43 @@ return noscriptUtil.service ? {
   },
   
   Hacks: {
-  
+    allowLocalLinks: function() {
+      let ns = noscriptOverlay.ns;
+      if (ns.geckoVersionCheck("30") >= 0) return;
+      
+      if ("urlSecurityCheck" in window) {
+        let usc = window.urlSecurityCheck;
+        window.urlSecurityCheck = function(aURL, aPrincipal, aFlags) {
+          if (!ns.checkLocalLink(aURL, aPrincipal)) {
+            usc.apply(this, arguments);
+          }
+        }
+      }
+      if ("handleLinkClick" in window) {
+        let hlc = window.handleLinkClick;
+        window.handleLinkClick = function(ev, href, linkNode) {
+          let ret = hlc.apply(this, arguments);
+          if (!ret && ns.checkLocalLink(linkNode.href, linkNode.nodePrincipal)) {
+            try {
+              let w = ev.view.open("about:blank", linkNode.target || "_self");
+              w.location.href = linkNode.href;
+              ev.preventDefault();
+              ret = true;
+            } catch (e) {
+             ret = false;
+            }
+          }
+          return ret;
+        }
+      }
+    },
     pdfDownload: function() {
       if (typeof(mouseClick) != "function") return;
       var tb = getBrowser();
       tb.removeEventListener("click", mouseClick, true);
       tb.addEventListener("click", mouseClick, false);
     },
-    
+  
     torButton: function() {
       if ("torbutton_update_tags" in window && typeof(window.torbutton_update_tags) == "function") {
         // we make TorButton aware that we could have a part in suppressing JavaScript on the browser
